@@ -400,8 +400,8 @@ static void con_ic_connection_send_event(ConIcConnection *connection,
 					 const char *bearer,
 					 ConIcConnectionStatus status)
 {
-	g_debug("con_ic_connection_send_event(%p, %s, %s, %i)",
-		(void *)connection, id, bearer, status);
+	/*g_debug("con_ic_connection_send_event(%p, %s, %s, %i)",
+		(void *)connection, id, bearer, status);*/
 	
 	ConIcConnectionEvent *event = con_ic_connection_event_new();
 
@@ -442,6 +442,8 @@ static DBusHandlerResult con_ic_connection_handle_icd_message(DBusConnection *c,
 		status = CON_IC_STATUS_CONNECTED;
 	} else if (strcmp(state, "NETWORKUP") == 0) {
 		status = CON_IC_STATUS_NETWORK_UP;
+	} else if (strcmp(state, "DISCONNECTING") == 0) {
+		status = CON_IC_STATUS_DISCONNECTING;
 	} else {
 		goto exit;
 	}
@@ -493,6 +495,7 @@ static DBusHandlerResult con_ic_connection_handle_icd_message(DBusConnection *c,
 		  break;
 
 		case CON_IC_STATUS_DISCONNECTED:		
+		case CON_IC_STATUS_DISCONNECTING:
 		  /* only the first IAP connection will be tracked */
 		  if (connection->internal_status == CONNECTED &&
 		      connection->iap_id != NULL &&
@@ -505,11 +508,15 @@ static DBusHandlerResult con_ic_connection_handle_icd_message(DBusConnection *c,
 							connection->bearer,
 							status);
 
-			connection->internal_status = DISCONNECTED;
-			g_free(connection->iap_id);
-			connection->iap_id = NULL;
-			g_free(connection->bearer);
-			connection->bearer = NULL;
+			if (status == CON_IC_STATUS_DISCONNECTING)
+				connection->internal_status = DISCONNECTING;
+			else {
+				connection->internal_status = DISCONNECTED;
+				g_free(connection->iap_id);
+				connection->iap_id = NULL;
+				g_free(connection->bearer);
+				connection->bearer = NULL;
+			}
 		  }
 		  break;
 
